@@ -32,28 +32,38 @@ GslModule*
 gsl_module_new (const GslClass *klass,
 		gpointer        user_data)
 {
-  OpNode *node;
+  EngineNode *node;
   guint i;
 
   g_return_val_if_fail (klass != NULL, NULL);
-  g_return_val_if_fail (klass->process != NULL, NULL);
+  g_return_val_if_fail (klass->process != NULL || klass->process_defer != NULL, NULL);
+  if (klass->process_defer)
+    {
+      g_warning ("%s: Delay cycle processing not yet implemented", G_STRLOC);
+      return NULL;
+    }
   
-  node = gsl_new_struct0 (OpNode, 1);
+  node = gsl_new_struct0 (EngineNode, 1);
   
   /* setup GslModule */
   node->module.klass = klass;
   node->module.user_data = user_data;
-  node->module.istreams = klass->n_istreams ? gsl_new_struct0 (GslIStream, OP_NODE_N_ISTREAMS (node)) : NULL;
-  node->module.ostreams = _gsl_alloc_ostreams (OP_NODE_N_OSTREAMS (node));
+  node->module.istreams = klass->n_istreams ? gsl_new_struct0 (GslIStream, ENGINE_NODE_N_ISTREAMS (node)) : NULL;
+  node->module.jstreams = klass->n_jstreams ? gsl_new_struct0 (GslJStream, ENGINE_NODE_N_JSTREAMS (node)) : NULL;
+  node->module.ostreams = _engine_alloc_ostreams (ENGINE_NODE_N_OSTREAMS (node));
   
-  /* setup OpNode */
-  node->inputs = OP_NODE_N_ISTREAMS (node) ? gsl_new_struct0 (OpInput, OP_NODE_N_ISTREAMS (node)) : NULL;
-  node->outputs = OP_NODE_N_OSTREAMS (node) ? gsl_new_struct0 (OpOutput, OP_NODE_N_OSTREAMS (node)) : NULL;
+  /* setup EngineNode */
+  node->inputs = ENGINE_NODE_N_ISTREAMS (node) ? gsl_new_struct0 (EngineInput, ENGINE_NODE_N_ISTREAMS (node)) : NULL;
+  node->jinputs = ENGINE_NODE_N_JSTREAMS (node) ? gsl_new_struct0 (EngineJInput*, ENGINE_NODE_N_JSTREAMS (node)) : NULL;
+  node->outputs = ENGINE_NODE_N_OSTREAMS (node) ? gsl_new_struct0 (EngineOutput, ENGINE_NODE_N_OSTREAMS (node)) : NULL;
   node->output_nodes = NULL;
   node->integrated = FALSE;
   gsl_rec_mutex_init (&node->rec_mutex);
-  for (i = 0; i < OP_NODE_N_OSTREAMS (node); i++)
-    node->outputs[i].buffer = node->module.ostreams[i].values;
+  for (i = 0; i < ENGINE_NODE_N_OSTREAMS (node); i++)
+    {
+      node->outputs[i].buffer = node->module.ostreams[i].values;
+      node->module.ostreams[i].sub_sample_pattern = gsl_engine_sub_sample_test (node->module.ostreams[i].values);
+    }
   node->flow_jobs = NULL;
   node->fjob_first = NULL;
   node->fjob_last = NULL;
@@ -76,7 +86,7 @@ gsl_module_tick_stamp (GslModule *module)
 {
   g_return_val_if_fail (module != NULL, 0);
   
-  return OP_NODE (module)->counter;
+  return ENGINE_NODE (module)->counter;
 }
 
 /**
@@ -94,8 +104,8 @@ gsl_job_integrate (GslModule *module)
   g_return_val_if_fail (module != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_INTEGRATE;
-  job->data.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_INTEGRATE;
+  job->data.node = ENGINE_NODE (module);
   
   return job;
 }
@@ -105,7 +115,7 @@ gsl_job_integrate (GslModule *module)
  * @module: The module to discard
  * @Returns: New job suitable for gsl_trans_add()
  *
- * Create a new transaction job which remove @module from the
+ * Create a new transaction job which removes @module from the
  * engine and destroys it.
  */
 GslJob*
@@ -116,8 +126,8 @@ gsl_job_discard (GslModule *module)
   g_return_val_if_fail (module != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_DISCARD;
-  job->data.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_DISCARD;
+  job->data.node = ENGINE_NODE (module);
   
   return job;
 }
@@ -149,10 +159,33 @@ gsl_job_connect (GslModule *src_module,
   g_return_val_if_fail (dest_istream < dest_module->klass->n_istreams, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_CONNECT;
-  job->data.connection.dest_node = OP_NODE (dest_module);
-  job->data.connection.dest_istream = dest_istream;
-  job->data.connection.src_node = OP_NODE (src_module);
+  job->job_id = ENGINE_JOB_ICONNECT;
+  job->data.connection.dest_node = ENGINE_NODE (dest_module);
+  job->data.connection.dest_ijstream = dest_istream;
+  job->data.connection.src_node = ENGINE_NODE (src_module);
+  job->data.connection.src_ostream = src_ostream;
+  
+  return job;
+}
+
+GslJob*
+gsl_job_jconnect (GslModule *src_module,
+		  guint      src_ostream,
+		  GslModule *dest_module,
+		  guint      dest_jstream)
+{
+  GslJob *job;
+  
+  g_return_val_if_fail (src_module != NULL, NULL);
+  g_return_val_if_fail (src_ostream < src_module->klass->n_ostreams, NULL);
+  g_return_val_if_fail (dest_module != NULL, NULL);
+  g_return_val_if_fail (dest_jstream < dest_module->klass->n_jstreams, NULL);
+  
+  job = gsl_new_struct0 (GslJob, 1);
+  job->job_id = ENGINE_JOB_JCONNECT;
+  job->data.connection.dest_node = ENGINE_NODE (dest_module);
+  job->data.connection.dest_ijstream = dest_jstream;
+  job->data.connection.src_node = ENGINE_NODE (src_module);
   job->data.connection.src_ostream = src_ostream;
   
   return job;
@@ -178,15 +211,38 @@ gsl_job_disconnect (GslModule *dest_module,
   g_return_val_if_fail (dest_istream < dest_module->klass->n_istreams, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_DISCONNECT;
-  job->data.connection.dest_node = OP_NODE (dest_module);
-  job->data.connection.dest_istream = dest_istream;
+  job->job_id = ENGINE_JOB_IDISCONNECT;
+  job->data.connection.dest_node = ENGINE_NODE (dest_module);
+  job->data.connection.dest_ijstream = dest_istream;
   job->data.connection.src_node = NULL;
   job->data.connection.src_ostream = ~0;
   
   return job;
 }
 
+GslJob*
+gsl_job_jdisconnect (GslModule *dest_module,
+		     guint      dest_jstream,
+		     GslModule *src_module,
+		     guint	src_ostream)
+{
+  GslJob *job;
+  
+  g_return_val_if_fail (dest_module != NULL, NULL);
+  g_return_val_if_fail (dest_jstream < dest_module->klass->n_jstreams, NULL);
+  g_return_val_if_fail (src_module != NULL, NULL);
+  g_return_val_if_fail (src_ostream < src_module->klass->n_ostreams, NULL);
+  
+  job = gsl_new_struct0 (GslJob, 1);
+  job->job_id = ENGINE_JOB_JDISCONNECT;
+  job->data.connection.dest_node = ENGINE_NODE (dest_module);
+  job->data.connection.dest_ijstream = dest_jstream;
+  job->data.connection.src_node = ENGINE_NODE (src_module);
+  job->data.connection.src_ostream = src_ostream;
+  
+  return job;
+}
+
 GslJob*
 gsl_job_set_consumer (GslModule *module,
 		      gboolean   is_toplevel_consumer)
@@ -196,8 +252,8 @@ gsl_job_set_consumer (GslModule *module,
   g_return_val_if_fail (module != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = is_toplevel_consumer ? GSL_JOB_SET_CONSUMER : GSL_JOB_UNSET_CONSUMER;
-  job->data.node = OP_NODE (module);
+  job->job_id = is_toplevel_consumer ? ENGINE_JOB_SET_CONSUMER : ENGINE_JOB_UNSET_CONSUMER;
+  job->data.node = ENGINE_NODE (module);
   
   return job;
 }
@@ -237,8 +293,8 @@ gsl_job_access (GslModule    *module,
   g_return_val_if_fail (access_func != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = GSL_JOB_ACCESS;
-  job->data.access.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_ACCESS;
+  job->data.access.node = ENGINE_NODE (module);
   job->data.access.access_func = access_func;
   job->data.access.data = data;
   job->data.access.free_func = free_func;
@@ -257,21 +313,21 @@ gsl_flow_job_access (GslModule    *module,
 		     GslFreeFunc   free_func)
 {
   GslJob *job;
-  GslFlowJob *fjob;
+  EngineFlowJob *fjob;
 
   g_return_val_if_fail (module != NULL, NULL);
   g_return_val_if_fail (access_func != NULL, NULL);
   
-  fjob = (GslFlowJob*) gsl_new_struct0 (GslFlowJobAccess, 1);
-  fjob->fjob_id = GSL_FLOW_JOB_ACCESS;
+  fjob = (EngineFlowJob*) gsl_new_struct0 (EngineFlowJobAccess, 1);
+  fjob->fjob_id = ENGINE_FLOW_JOB_ACCESS;
   fjob->any.tick_stamp = tick_stamp;
   fjob->access.access_func = access_func;
   fjob->access.data = data;
   fjob->access.free_func = free_func;
 
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = GSL_JOB_FLOW_JOB;
-  job->data.flow_job.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_FLOW_JOB;
+  job->data.flow_job.node = ENGINE_NODE (module);
   job->data.flow_job.fjob = fjob;
   
   return job;
@@ -282,17 +338,17 @@ gsl_flow_job_suspend (GslModule *module,
 		      guint64    tick_stamp)
 {
   GslJob *job;
-  GslFlowJob *fjob;
+  EngineFlowJob *fjob;
   
   g_return_val_if_fail (module != NULL, NULL);
   
-  fjob = (GslFlowJob*) gsl_new_struct0 (GslFlowJobAny, 1);
-  fjob->fjob_id = GSL_FLOW_JOB_SUSPEND;
+  fjob = (EngineFlowJob*) gsl_new_struct0 (EngineFlowJobAny, 1);
+  fjob->fjob_id = ENGINE_FLOW_JOB_SUSPEND;
   fjob->any.tick_stamp = tick_stamp;
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = GSL_JOB_FLOW_JOB;
-  job->data.flow_job.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_FLOW_JOB;
+  job->data.flow_job.node = ENGINE_NODE (module);
   job->data.flow_job.fjob = fjob;
   
   return job;
@@ -303,17 +359,17 @@ gsl_flow_job_resume (GslModule *module,
 		     guint64    tick_stamp)
 {
   GslJob *job;
-  GslFlowJob *fjob;
+  EngineFlowJob *fjob;
   
   g_return_val_if_fail (module != NULL, NULL);
   
-  fjob = (GslFlowJob*) gsl_new_struct0 (GslFlowJobAny, 1);
-  fjob->fjob_id = GSL_FLOW_JOB_RESUME;
+  fjob = (EngineFlowJob*) gsl_new_struct0 (EngineFlowJobAny, 1);
+  fjob->fjob_id = ENGINE_FLOW_JOB_RESUME;
   fjob->any.tick_stamp = tick_stamp;
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = GSL_JOB_FLOW_JOB;
-  job->data.flow_job.node = OP_NODE (module);
+  job->job_id = ENGINE_JOB_FLOW_JOB;
+  job->data.flow_job.node = ENGINE_NODE (module);
   job->data.flow_job.fjob = fjob;
   
   return job;
@@ -355,11 +411,11 @@ gsl_flow_job_resume (GslModule *module,
  * determine whether processing is currently necessary.
  */
 GslJob*
-gsl_job_add_poll (GslPollFunc      poll_func,
-		  gpointer         data,
-		  GslFreeFunc      free_func,
-		  guint            n_fds,
-		  const GslPollFD *fds)
+gsl_job_add_poll (GslPollFunc    poll_func,
+		  gpointer       data,
+		  GslFreeFunc    free_func,
+		  guint          n_fds,
+		  const GPollFD *fds)
 {
   GslJob *job;
   
@@ -368,7 +424,7 @@ gsl_job_add_poll (GslPollFunc      poll_func,
     g_return_val_if_fail (fds != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_ADD_POLL;
+  job->job_id = ENGINE_JOB_ADD_POLL;
   job->data.poll.poll_func = poll_func;
   job->data.poll.data = data;
   job->data.poll.n_fds = n_fds;
@@ -395,7 +451,7 @@ gsl_job_remove_poll (GslPollFunc poll_func,
   g_return_val_if_fail (poll_func != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_REMOVE_POLL;
+  job->job_id = ENGINE_JOB_REMOVE_POLL;
   job->data.poll.poll_func = poll_func;
   job->data.poll.data = data;
   job->data.poll.free_func = NULL;
@@ -421,7 +477,7 @@ gsl_job_debug (const gchar *debug)
   g_return_val_if_fail (debug != NULL, NULL);
   
   job = gsl_new_struct0 (GslJob, 1);
-  job->job_id = OP_JOB_DEBUG;
+  job->job_id = ENGINE_JOB_DEBUG;
   job->data.debug = g_strdup (debug);
   
   return job;
@@ -494,7 +550,7 @@ gsl_trans_commit (GslTrans *trans)
   if (trans->jobs_head)
     {
       trans->comitted = TRUE;
-      op_com_enqueue_trans (trans);
+      _engine_enqueue_trans (trans);
       wakeup_master ();
     }
   else
@@ -517,7 +573,7 @@ gsl_trans_dismiss (GslTrans *trans)
   g_return_if_fail (trans->comitted == FALSE);
   g_return_if_fail (trans->cqt_next == NULL);
   
-  _gsl_free_trans (trans);
+  _engine_free_trans (trans);
 
   gsl_engine_garbage_collect ();
 }
@@ -549,51 +605,6 @@ gsl_transact (GslJob *job,
 }
 
 
-/* --- debugging --- */
-static volatile GslEngineDebugLevel op_debug_levels = 0;
-
-void
-gsl_engine_debug_enable (GslEngineDebugLevel level)
-{
-  level |= op_debug_levels;
-  op_debug_levels = level;
-}
-
-void
-gsl_engine_debug_disable (GslEngineDebugLevel level)
-{
-  level = op_debug_levels & ~level;
-  op_debug_levels = level;
-}
-
-void
-_gsl_op_debug (GslEngineDebugLevel lvl,
-	       const gchar        *format,
-	       ...)
-{
-  if (lvl & op_debug_levels)
-    {
-      va_list var_args;
-      gchar *s, *l;
-      
-      switch (lvl)
-	{
-	case GSL_ENGINE_DEBUG_ENGINE:	l = "ENGINE";	break;
-	case GSL_ENGINE_DEBUG_JOBS:	l = "JOBS";	break;
-	case GSL_ENGINE_DEBUG_SCHED:	l = "SCHED";	break;
-	case GSL_ENGINE_DEBUG_MASTER:	l = "MASTER";	break;
-	case GSL_ENGINE_DEBUG_SLAVE:	l = "SLAVE";	break;
-	default:			l = "UNKNOWN";	break;
-	}
-      va_start (var_args, format);
-      s = g_strdup_vprintf (format, var_args);
-      va_end (var_args);
-      g_printerr (G_LOG_DOMAIN ": DEBUG_%s(%p): %s\n", l, gsl_thread_self (), s);
-      g_free (s);
-    }
-}
-
-
 /* --- initialization --- */
 static void
 slave (gpointer data)
@@ -624,6 +635,8 @@ static gboolean   gsl_engine_threaded = FALSE;
 static GslThread *master_thread = NULL;
 guint		gsl_externvar_bsize = 0;
 guint		gsl_externvar_sample_freq = 0;
+guint		gsl_externvar_sub_sample_mask = 0;
+guint		gsl_externvar_sub_sample_steps = 0;
 
 /**
  * gsl_engine_init
@@ -638,23 +651,28 @@ guint		gsl_externvar_sample_freq = 0;
 void
 gsl_engine_init (gboolean run_threaded,
 		 guint	  block_size,
-		 guint	  sample_freq)
+		 guint	  sample_freq,
+		 guint    sub_sample_mask)
 {
   g_return_if_fail (gsl_engine_initialized == FALSE);
   g_return_if_fail (block_size > 0 && block_size <= GSL_STREAM_MAX_VALUES);
   g_return_if_fail (sample_freq > 0);
+  g_return_if_fail (sub_sample_mask < block_size);
+  g_return_if_fail ((sub_sample_mask & (sub_sample_mask + 1)) == 0);	/* power of 2 */
   
   gsl_engine_initialized = TRUE;
   gsl_engine_threaded = run_threaded;
   gsl_externvar_bsize = block_size;
   gsl_externvar_sample_freq = sample_freq;
+  gsl_externvar_sub_sample_mask = sub_sample_mask << 2;	/* shift out sizeof (float) alignment */
+  gsl_externvar_sub_sample_steps = sub_sample_mask + 1;
   _gsl_tick_stamp_set_leap (block_size);
   
-  OP_DEBUG (GSL_ENGINE_DEBUG_ENGINE, "initialization: threaded=%s", gsl_engine_threaded ? "TRUE" : "FALSE");
+  ENG_DEBUG ("initialization: threaded=%s", gsl_engine_threaded ? "TRUE" : "FALSE");
   
   if (gsl_engine_threaded)
     {
-      master_thread = gsl_thread_new (_gsl_master_thread, NULL);
+      master_thread = gsl_thread_new (_engine_master_thread, NULL);
       if (0)
 	gsl_thread_new (slave, NULL);
     }
@@ -674,7 +692,7 @@ gsl_engine_prepare (GslEngineLoop *loop)
   g_return_val_if_fail (gsl_engine_initialized == TRUE, FALSE);
 
   if (!gsl_engine_threaded)
-    return _gsl_master_prepare (loop);
+    return _engine_master_prepare (loop);
   else
     {
       loop->timeout = -1;
@@ -693,7 +711,7 @@ gsl_engine_check (const GslEngineLoop *loop)
     g_return_val_if_fail (loop->revents_filled == TRUE, FALSE);
   
   if (!gsl_engine_threaded)
-    return _gsl_master_check (loop);
+    return _engine_master_check (loop);
   else
     return FALSE;
 }
@@ -704,7 +722,7 @@ gsl_engine_dispatch (void)
   g_return_if_fail (gsl_engine_initialized == TRUE);
 
   if (!gsl_engine_threaded)
-    _gsl_master_dispatch ();
+    _engine_master_dispatch ();
 }
 
 /**
@@ -722,10 +740,10 @@ gsl_engine_wait_on_trans (void)
   
   /* non-threaded */
   if (!gsl_engine_threaded)
-    _gsl_master_dispatch_jobs ();
+    _engine_master_dispatch_jobs ();
 
   /* threaded */
-  op_com_wait_on_trans ();
+  _engine_wait_on_trans ();
   
   /* call all free() functions */
   gsl_engine_garbage_collect ();
