@@ -70,17 +70,24 @@ typedef gint32		GTime;
 typedef long int		gint64;
 typedef unsigned long int	guint64;
 #else
-typedef long long int		gint64;
+typedef long long int	gint64;
 typedef unsigned long long int  guint64;
 #endif
-typedef struct _GString		GString;
+typedef struct _GString GString;
+typedef struct _GDebugKey       GDebugKey;
+struct _GDebugKey
+{
+  const gchar *key;
+  guint  value;
+};
 typedef struct _GTimeVal        GTimeVal;
-
 struct _GTimeVal
 {
   glong tv_sec;
   glong tv_usec;
 };
+typedef gint            (*GCompareFunc)         (gconstpointer  a,
+                                                 gconstpointer  b);
 
 
 /* --- standard macros --- */
@@ -116,8 +123,11 @@ struct _GTimeVal
 #define G_MAXSHORT      SHRT_MAX
 #define G_MAXUSHORT     USHRT_MAX
 #define G_MININT        INT_MIN
+#define G_MININT64      ((gint64)  0x8000000000000000)
 #define G_MAXINT        INT_MAX
+#define G_MAXINT64      ((gint64)  0x7fffffffffffffff)
 #define G_MAXUINT       UINT_MAX
+#define G_MAXUINT64     ((guint64) 0xffffffffffffffff)
 #define G_MINLONG       LONG_MIN
 #define G_MAXLONG       LONG_MAX
 #define G_MAXULONG      ULONG_MAX
@@ -132,10 +142,11 @@ struct _GTimeVal
 #define	G_PATH_LENGTH	PATH_MAX
 #elif   defined (_PC_PATH_MAX)
 #define	G_PATH_LENGTH	sysconf(_PC_PATH_MAX)
-#else	
+#else
 #define G_PATH_LENGTH   2048
 #endif
 
+#define G_N_ELEMENTS(arr)          (sizeof (arr) / sizeof ((arr)[0]))
 #define G_STRINGIFY(macro_or_string)    G_STRINGIFY_ARG (macro_or_string)
 #define G_STRINGIFY_ARG(contents)       #contents
 #if  defined __GNUC__ && !defined __cplusplus
@@ -192,8 +203,8 @@ struct _GTimeVal
 #define GUINT16_FROM_BE(val)    (GUINT16_TO_BE (val))
 #define GUINT32_FROM_LE(val)    (GUINT32_TO_LE (val))
 #define GUINT32_FROM_BE(val)    (GUINT32_TO_BE (val))
-       
-       
+
+
 #define g_memmove memmove
 #define g_assert  GSL_ASSERT
 #define g_assert_not_reached()	g_assert(!G_STRLOC": should not be reached")
@@ -207,7 +218,7 @@ struct _GTimeVal
 # undef alloca
 # define alloca(size)   __builtin_alloca (size)
 #elif defined (GLIB_HAVE_ALLOCA_H)
-/* a native and working alloca.h is there */ 
+/* a native and working alloca.h is there */
 # include <alloca.h>
 #else /* !__GNUC__ && !GLIB_HAVE_ALLOCA_H */
 # ifdef _MSC_VER
@@ -263,6 +274,15 @@ g_message (const gchar *format,
   va_end (args);
 }
 static inline void
+g_critical (const gchar *format,
+	    ...)
+{
+  va_list args;
+  va_start (args, format);
+  gsl_g_log ("**CRITICAL**", format, args);
+  va_end (args);
+}
+static inline void
 g_warning (const gchar *format,
 	   ...)
 {
@@ -383,8 +403,19 @@ g_trash_stack_height (GTrashStack **stack_p)
 
 
 
-/* --- sick defines --- */
-typedef struct { int fd; short events, revents; } GPollFD;
+/* --- GPollFD (for poll(2)) --- */
+#define G_IO_IN      (0x0001 /* There is data to read */)
+#define G_IO_PRI     (0x0002 /* There is urgent data to read */)
+#define G_IO_OUT     (0x0004 /* Writing now will not block */)
+#define G_IO_ERR     (0x0008 /* Error condition */)
+#define G_IO_HUP     (0x0010 /* Hung up */)
+#define G_IO_NVAL    (0x0020 /* Invalid request: fd not open */)
+typedef struct
+{
+  gint    fd;
+  gushort events;
+  gushort revents;
+} GPollFD;
 
 
 
@@ -403,10 +434,10 @@ typedef struct { int fd; short events, revents; } GPollFD;
 #define	g_usleep		gsl_g_usleep
 #define	g_strerror		gsl_g_strerror
 #define	g_convert		gsl_g_convert
-#define g_direct_hash	 	gsl_g_direct_hash 
-#define g_direct_equal 		gsl_g_direct_equal 
-#define g_str_equal 		gsl_g_str_equal 
-#define g_str_hash 		gsl_g_str_hash 
+#define g_direct_hash	 	gsl_g_direct_hash
+#define g_direct_equal 		gsl_g_direct_equal
+#define g_str_equal 		gsl_g_str_equal
+#define g_str_hash 		gsl_g_str_hash
 #define g_strtod		gsl_g_strtod
 #define g_stpcpy		gsl_g_stpcpy
 #define	g_strescape		gsl_g_strescape
@@ -467,15 +498,17 @@ gboolean g_path_is_absolute (const gchar *file_name);
 
 
 /* --- configure stuff!!! --- */
-#define	G_BYTE_ORDER G_LITTLE_ENDIAN
+#ifdef WORDS_BIGENDIAN
+#define G_BYTE_ORDER G_BIG_ENDIAN
+#else
+#define G_BYTE_ORDER G_LITTLE_ENDIAN
+#endif
+
 /* #define	GLIB_HAVE_STPCPY	1 */
 /* Define G_VA_COPY() to do the right thing for copying va_list variables.
  * glibconfig.h may have already defined G_VA_COPY as va_copy or __va_copy.
  */
 #if !defined (G_VA_COPY)
-# ifdef __va_copy
-#  define G_VA_COPY __va_copy
-# else
 #  if defined (__GNUC__) && ( defined (__PPC__) || defined (__s390__) ) && (defined (_CALL_SYSV) || defined (_WIN32) || defined (__s390__) )
 #    define G_VA_COPY(ap1, ap2)   (*(ap1) = *(ap2))
 #  elif defined (G_VA_COPY_AS_ARRAY)
@@ -483,7 +516,6 @@ gboolean g_path_is_absolute (const gchar *file_name);
 #  else /* va_list is a pointer */
 #    define G_VA_COPY(ap1, ap2)   ((ap1) = (ap2))
 #  endif /* va_list is a pointer */
-# endif
 #endif /* !G_VA_COPY */
 
 
@@ -554,19 +586,19 @@ typedef guint           (*GHashFunc)            (gconstpointer  key);
 typedef gboolean        (*GEqualFunc)           (gconstpointer  a,
 						 gconstpointer  b);
 typedef void            (*GDestroyNotify)       (gpointer       data);
-#define g_hash_table_new               	gsl_g_hash_table_new               
-#define g_hash_table_new_full          	gsl_g_hash_table_new_full          
-#define g_hash_table_destroy           	gsl_g_hash_table_destroy           
-#define g_hash_table_insert            	gsl_g_hash_table_insert            
-#define g_hash_table_replace           	gsl_g_hash_table_replace           
-#define g_hash_table_remove            	gsl_g_hash_table_remove            
-#define g_hash_table_steal             	gsl_g_hash_table_steal             
-#define g_hash_table_lookup            	gsl_g_hash_table_lookup            
-#define g_hash_table_lookup_extended   	gsl_g_hash_table_lookup_extended   
-#define g_hash_table_foreach           	gsl_g_hash_table_foreach           
-#define g_hash_table_foreach_remove    	gsl_g_hash_table_foreach_remove    
-#define g_hash_table_foreach_steal     	gsl_g_hash_table_foreach_steal     
-#define g_hash_table_size              	gsl_g_hash_table_size              
+#define g_hash_table_new               	gsl_g_hash_table_new
+#define g_hash_table_new_full          	gsl_g_hash_table_new_full
+#define g_hash_table_destroy           	gsl_g_hash_table_destroy
+#define g_hash_table_insert            	gsl_g_hash_table_insert
+#define g_hash_table_replace           	gsl_g_hash_table_replace
+#define g_hash_table_remove            	gsl_g_hash_table_remove
+#define g_hash_table_steal             	gsl_g_hash_table_steal
+#define g_hash_table_lookup            	gsl_g_hash_table_lookup
+#define g_hash_table_lookup_extended   	gsl_g_hash_table_lookup_extended
+#define g_hash_table_foreach           	gsl_g_hash_table_foreach
+#define g_hash_table_foreach_remove    	gsl_g_hash_table_foreach_remove
+#define g_hash_table_foreach_steal     	gsl_g_hash_table_foreach_steal
+#define g_hash_table_size              	gsl_g_hash_table_size
 GHashTable* g_hash_table_new               (GHashFunc       hash_func,
 					    GEqualFunc      key_equal_func);
 GHashTable* g_hash_table_new_full          (GHashFunc       hash_func,
@@ -634,7 +666,7 @@ typedef enum
 typedef enum
 {
   G_TOKEN_EOF			=   0,
-  
+
   G_TOKEN_LEFT_PAREN		= '(',
   G_TOKEN_RIGHT_PAREN		= ')',
   G_TOKEN_LEFT_CURLY		= '{',
@@ -643,11 +675,11 @@ typedef enum
   G_TOKEN_RIGHT_BRACE		= ']',
   G_TOKEN_EQUAL_SIGN		= '=',
   G_TOKEN_COMMA			= ',',
-  
+
   G_TOKEN_NONE			= 256,
-  
+
   G_TOKEN_ERROR,
-  
+
   G_TOKEN_CHAR,
   G_TOKEN_BINARY,
   G_TOKEN_OCTAL,
@@ -655,11 +687,11 @@ typedef enum
   G_TOKEN_HEX,
   G_TOKEN_FLOAT,
   G_TOKEN_STRING,
-  
+
   G_TOKEN_SYMBOL,
   G_TOKEN_IDENTIFIER,
   G_TOKEN_IDENTIFIER_NULL,
-  
+
   G_TOKEN_COMMENT_SINGLE,
   G_TOKEN_COMMENT_MULTI,
   G_TOKEN_LAST
@@ -680,10 +712,10 @@ union	_GTokenValue
 };
 struct	_GScannerConfig
 {
-  gchar		*cset_skip_characters;		/* default: " \t\n" */
-  gchar		*cset_identifier_first;
-  gchar		*cset_identifier_nth;
-  gchar		*cpair_comment_single;		/* default: "#\n" */
+  const gchar	*cset_skip_characters;		/* default: " \t\n" */
+  const gchar	*cset_identifier_first;
+  const gchar	*cset_identifier_nth;
+  const gchar	*cpair_comment_single;		/* default: "#\n" */
   guint		case_sensitive : 1;
   guint		skip_comment_multi : 1;		/* C like comment */
   guint		skip_comment_single : 1;	/* single line comment */
@@ -730,27 +762,27 @@ struct	_GScanner
   guint			scope_id;
   GScannerMsgFunc	msg_handler;
 };
-#define g_scanner_new			   gsl_g_scanner_new			
-#define g_scanner_destroy		   gsl_g_scanner_destroy		
-#define g_scanner_input_file		   gsl_g_scanner_input_file		
-#define g_scanner_sync_file_offset	   gsl_g_scanner_sync_file_offset	
-#define g_scanner_input_text		   gsl_g_scanner_input_text		
-#define g_scanner_get_next_token	   gsl_g_scanner_get_next_token	
-#define g_scanner_peek_next_token	   gsl_g_scanner_peek_next_token	
-#define g_scanner_cur_token		   gsl_g_scanner_cur_token		
-#define g_scanner_cur_value		   gsl_g_scanner_cur_value		
-#define g_scanner_cur_line		   gsl_g_scanner_cur_line		
-#define g_scanner_cur_position		   gsl_g_scanner_cur_position		
-#define g_scanner_eof			   gsl_g_scanner_eof			
-#define g_scanner_set_scope		   gsl_g_scanner_set_scope		
-#define g_scanner_scope_add_symbol	   gsl_g_scanner_scope_add_symbol	
-#define g_scanner_scope_remove_symbol	   gsl_g_scanner_scope_remove_symbol	
-#define g_scanner_scope_lookup_symbol	   gsl_g_scanner_scope_lookup_symbol	
-#define g_scanner_scope_foreach_symbol	   gsl_g_scanner_scope_foreach_symbol	
-#define g_scanner_lookup_symbol		   gsl_g_scanner_lookup_symbol		
-#define g_scanner_unexp_token		   gsl_g_scanner_unexp_token		
-#define g_scanner_error			   gsl_g_scanner_error			
-#define g_scanner_warn			   gsl_g_scanner_warn			
+#define g_scanner_new			   gsl_g_scanner_new
+#define g_scanner_destroy		   gsl_g_scanner_destroy
+#define g_scanner_input_file		   gsl_g_scanner_input_file
+#define g_scanner_sync_file_offset	   gsl_g_scanner_sync_file_offset
+#define g_scanner_input_text		   gsl_g_scanner_input_text
+#define g_scanner_get_next_token	   gsl_g_scanner_get_next_token
+#define g_scanner_peek_next_token	   gsl_g_scanner_peek_next_token
+#define g_scanner_cur_token		   gsl_g_scanner_cur_token
+#define g_scanner_cur_value		   gsl_g_scanner_cur_value
+#define g_scanner_cur_line		   gsl_g_scanner_cur_line
+#define g_scanner_cur_position		   gsl_g_scanner_cur_position
+#define g_scanner_eof			   gsl_g_scanner_eof
+#define g_scanner_set_scope		   gsl_g_scanner_set_scope
+#define g_scanner_scope_add_symbol	   gsl_g_scanner_scope_add_symbol
+#define g_scanner_scope_remove_symbol	   gsl_g_scanner_scope_remove_symbol
+#define g_scanner_scope_lookup_symbol	   gsl_g_scanner_scope_lookup_symbol
+#define g_scanner_scope_foreach_symbol	   gsl_g_scanner_scope_foreach_symbol
+#define g_scanner_lookup_symbol		   gsl_g_scanner_lookup_symbol
+#define g_scanner_unexp_token		   gsl_g_scanner_unexp_token
+#define g_scanner_error			   gsl_g_scanner_error
+#define g_scanner_warn			   gsl_g_scanner_warn
 GScanner*	g_scanner_new			(const GScannerConfig *config_templ);
 void		g_scanner_destroy		(GScanner	*scanner);
 void		g_scanner_input_file		(GScanner	*scanner,
@@ -814,10 +846,17 @@ void		g_scanner_warn			(GScanner	*scanner,
 }
 #endif /* __cplusplus */
 
-#ifdef GSL_WANT_ARTS_THREADS
-#include "gslartsthreads.h"
+#include <gsl/gslconfig.h>
+#if GSL_USE_ARTS_THREADS
+#include <gsl/gslartsthreads.h>
 #endif
 
+/* if inline is defined in C++ mode, it can cause a lot of grief with C++ */
+/* header files so we need to undefine it here again */
+
+#if defined(__cplusplus) && defined(inline)
+#undef inline
+#endif
 
 #endif /* __GSL_GLIB_H__ */ /* vim: set ts=8 sw=2 sts=2: */
 
