@@ -1,7 +1,7 @@
     /*
 
-    Copyright (C) 2000 Stefan Westerfeld
-                       stefan@space.twc.de
+    Copyright (C) 2000-2002 Stefan Westerfeld
+                            stefan@space.twc.de
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -12,7 +12,7 @@
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
-   
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -48,9 +48,10 @@ public:
 	static bool gslDataCalculated;
 
 	/* static check function */
-	static gboolean gslCheck(gpointer data, guint n_values, glong *timeout_p,
-                         	 guint n_fds, const GslPollFD *fds,
-						 	 gboolean revents_filled)
+	static gboolean gslCheck(gpointer /* data */, guint /* n_values */,
+							 glong*  /* timeout_p */,
+                         	 guint /* n_fds */, const GPollFD* /* fds */,
+						 	 gboolean /* revents_filled */)
 	{
 		return waitOnTransNeedData;
 	}
@@ -86,7 +87,7 @@ public:
 
 		while(!gslDataCalculated && gsl_engine_check(&loop))
 			gsl_engine_dispatch();
-	
+
 		gslDataCalculated = false;
 		waitOnTransNeedData = false;
 
@@ -121,44 +122,9 @@ namespace Arts { extern void *gslGlobalMutexTable; }
 using namespace std;
 using namespace Arts;
 
-// well, this was tuneable once...
-
-static unsigned long requestSize()
-{
-	unsigned long reqSize = 0;
-	if(!reqSize)
-		reqSize = AudioSubSystem::the()->fragmentSize()/4;
-
-	return reqSize;
-}
-
-// ----------- SynthBuffer -------------
-
-SynthBuffer::SynthBuffer(float initialvalue, unsigned long size)
-{
-	this->size = size;
-	data = new float[size];
-
-	setValue(initialvalue);
-
-	position = 0;
-	needread = 0;
-}
-
-void SynthBuffer::setValue(float value)
-{
-	unsigned long i;
-	for(i=0;i<size;i++) data[i] = value;
-}
-
-SynthBuffer::~SynthBuffer()
-{
-	delete[] data;
-}
-
 // ----------- Port -----------
 
-Port::Port(string name, void *ptr, long flags, StdScheduleNode* parent)
+Port::Port(const string& name, void *ptr, long flags, StdScheduleNode* parent)
 	: _name(name), _ptr(ptr), _flags((AttributeType)flags),
 	  parent(parent), _dynamicPort(false)
 {
@@ -239,20 +205,19 @@ void Port::setPtr(void *ptr)
 
 // ------- AudioPort ---------
 
-AudioPort::AudioPort(string name, void *ptr, long flags,StdScheduleNode *parent)
+AudioPort::AudioPort(const string& name,
+		             void *ptr, long flags,StdScheduleNode *parent)
 		: Port(name,ptr,flags,parent)
 {
-	position = 0;
 	destcount = 0;
 	sourcemodule = 0;
 	source = 0;
 	gslIsConstant = false;
-	lbuffer = buffer = new SynthBuffer(0.0, requestSize());
 }
 
 AudioPort::~AudioPort()
 {
-	delete lbuffer;
+	//
 }
 
 AudioPort *AudioPort::audioPort()
@@ -264,7 +229,8 @@ void AudioPort::setFloatValue(float f)
 {
 	gslIsConstant = true;
 	gslConstantValue = f;
-	buffer->setValue(f);
+
+	parent->_connectionCountChanged = true;
 }
 
 void AudioPort::connect(Port *psource)
@@ -273,8 +239,6 @@ void AudioPort::connect(Port *psource)
 	assert(source);
 	addAutoDisconnect(psource);
 
-	buffer = source->buffer;
-	position = buffer->position;
 	source->destcount++;
 	sourcemodule = source->parent;
 
@@ -284,6 +248,8 @@ void AudioPort::connect(Port *psource)
 										 parent->gslModule,
 										 gslEngineChannel));
 	gsl_trans_commit(trans);
+
+	source->parent->_connectionCountChanged = parent->_connectionCountChanged = true;
 }
 
 void AudioPort::disconnect(Port *psource)
@@ -295,14 +261,10 @@ void AudioPort::disconnect(Port *psource)
 	assert(sourcemodule == source->parent);
 	sourcemodule = 0;
 
-	// skip the remaining stuff in the buffer
-	read(buffer->position - position);
+	source->parent->_connectionCountChanged = parent->_connectionCountChanged = true;
 	source->destcount--;
 	source = 0;
 
-	position = lbuffer->position;
-	buffer = lbuffer;
-
 	// GSL disconnect
 	GslTrans *trans = gsl_trans_open();
 	gsl_trans_add(trans, gsl_job_disconnect(parent->gslModule,
@@ -312,7 +274,8 @@ void AudioPort::disconnect(Port *psource)
 
 // --------- MultiPort ----------
 
-MultiPort::MultiPort(string name, void *ptr, long flags,StdScheduleNode *parent)
+MultiPort::MultiPort(const string& name,
+		             void *ptr, long flags,StdScheduleNode *parent)
 	: Port(name,ptr,flags,parent)
 {
 	conns = 0;
@@ -338,10 +301,10 @@ void MultiPort::initConns()
 	*(float ***)_ptr = conns;
 
 	long n = 0;
-	std::list<AudioPort *>::iterator i;
+	std::list<Part>::iterator i;
 	for(i = parts.begin();i != parts.end(); i++)
 	{
-		AudioPort *p = *i;
+		AudioPort *p = i->dest;
 		p->setPtr((void *)&conns[n++]);
 	}
 }
@@ -355,7 +318,12 @@ void MultiPort::connect(Port *port)
 	addAutoDisconnect(port);
 
 	dport = new AudioPort("_"+_name+string(sid),0,streamIn,parent);
-	parts.push_back(dport);
+
+	Part part;
+	part.src = (AudioPort *)port;
+	part.dest = dport;
+
+	parts.push_back(part);
 	initConns();
 
 	parent->addDynamicPort(dport);
@@ -367,13 +335,12 @@ void MultiPort::disconnect(Port *sport)
 	AudioPort *port = (AudioPort *)sport;
 	removeAutoDisconnect(sport);
 
-	std::list<AudioPort *>::iterator i;
+	std::list<Part>::iterator i;
 	for(i = parts.begin(); i != parts.end(); i++)
 	{
-		AudioPort *dport = *i;
-
-		if(dport->buffer == port->buffer)
+		if(i->src == port)
 		{
+			AudioPort *dport = i->dest;
 			parts.erase(i);
 			initConns();
 
@@ -420,9 +387,8 @@ void StdScheduleNode::gslProcess(GslModule *module, guint n_values)
 	arts_return_if_fail(node->module != 0);
 
 	GslMainLoop::gslDataCalculated = true;
-	//fprintf(stderr,"TODO: write process wrapper\n");
-	unsigned long j, donecycles = 0, cando = 0;
 
+	unsigned long j;
 	for(j=0;j<node->inConnCount;j++)
 	{
 		if(node->inConn[j]->gslIsConstant)
@@ -438,7 +404,7 @@ void StdScheduleNode::gslProcess(GslModule *module, guint n_values)
 	node->module->calculateBlock(n_values);
 }
 
-static void gslModuleFree(gpointer data, const GslClass *klass)
+static void gslModuleFree(gpointer /* data */, const GslClass *klass)
 {
 	gslMainLoop.freeGslClass(const_cast<GslClass *>(klass));
 }
@@ -488,7 +454,7 @@ void StdScheduleNode::rebuildConn()
 	/* since destroying the old module and creating a new one will destroy
 	 * all the connections, we need to restore them here
 	 */
-	int c;
+	unsigned int c;
 	for(c = 0; c < inConnCount; c++)
 	{
 		if(inConn[c]->source)
@@ -503,7 +469,7 @@ void StdScheduleNode::rebuildConn()
 	for(c = 0; c < outConnCount; c++)
 	{
 		std::list<Port *>::iterator ci;
-		
+
 		for(ci = outConn[c]->autoDisconnect.begin();
 			ci != outConn[c]->autoDisconnect.end(); ci++)
 		{
@@ -555,7 +521,6 @@ StdScheduleNode::StdScheduleNode(Object_skel *object, StdFlowSystem *flowSystem)
 	queryInitStreamFunc = 0;
 	inConn = outConn = 0;
 	inConnCount = outConnCount = 0;
-	Busy = BusyHit = NeedCycles = CanPerform = 0;
 }
 
 StdScheduleNode::~StdScheduleNode()
@@ -630,7 +595,7 @@ void StdScheduleNode::removeDynamicPort(Port *port)
 		{
 			ports.erase(i);
 			rebuildConn();
-			return;	
+			return;
 		}
 	}
 }
@@ -660,8 +625,6 @@ void StdScheduleNode::stop()
 void StdScheduleNode::requireFlow()
 {
 	// cout << "rf" << module->_interfaceName() << endl;
-	//request(requestSize());
-	/* GSL! flowSystem->schedule(requestSize()); */
 	flowSystem->updateStarted();
 	gslMainLoop.run();
 }
@@ -694,7 +657,7 @@ void StdScheduleNode::restart()
 	}
 }
 
-Port *StdScheduleNode::findPort(string name)
+Port *StdScheduleNode::findPort(const string& name)
 {
 	std::list<Port *>::iterator i;
 	for(i=ports.begin();i!=ports.end();i++)
@@ -732,7 +695,7 @@ void StdScheduleNode::virtualize(const std::string& port,
 	}
 }
 
-void StdScheduleNode::devirtualize(const std::string& port, 
+void StdScheduleNode::devirtualize(const std::string& port,
 		                           ScheduleNode *implNode,
 								   const std::string& implPort)
 {
@@ -816,7 +779,7 @@ AttributeType StdScheduleNode::queryFlags(const std::string& port)
 	return (AttributeType)0;
 }
 
-void StdScheduleNode::setFloatValue(const string& port, float value)           
+void StdScheduleNode::setFloatValue(const string& port, float value)
 {
 	AudioPort *p = findPort(port)->audioPort();
 
@@ -827,162 +790,35 @@ void StdScheduleNode::setFloatValue(const string& port, float value)
 	}
 }
 
-/* request a module to calculate a number of turns
-   will return -1 if busy, count that has been done otherwise */
-
-long StdScheduleNode::request(long amount)
+unsigned long StdScheduleNode::inputConnectionCount(const string& port)
 {
-	unsigned long in;
-	int have_in,need_more,have_done,have_done_total = 0;
+	unsigned long result = 0;
 
-	if(!running)
+	unsigned int c;
+	for(c = 0; c < inConnCount; c++)
 	{
-		arts_fatal("Calculating data on a module which was not started!\n"
-					"Start modules by calling module.start() "
-					"before connecting them to avoid this.");
-		// not reached
-	}
-
-	if(Busy) { BusyHit++; return(-1); }
-
-	Busy = 1;
-	if(NeedCycles < amount)
-	{
-		NeedCycles = amount;
-	}
-
-	// cout << "re" << module->_interfaceName() << endl;
-	//artsdebug("DSP %s: request of %d cycles\n",getClassName(),amount);
-	do
-	{
-		/* assume we can satisfy the request */
-		CanPerform = NeedCycles;
-
-		/* then, check wether the input channels supply enough data to do so. */
-
-		for(in=0;in<inConnCount;in++)
+		if(inConn[c]->name() == port)
 		{
-			have_in = inConn[in]->haveIn();
-
-			if(have_in < NeedCycles)
-			{
-				//artsdebug("connection %d indicates to have %d, "
-                //       "thats not enough\n", in,have_in);
-				/* if we can't calculate enough stuff due to a certain
-				   ingoing connection, go to the associated module and
-				   tell it that we need more data
-				*/
-				need_more = NeedCycles - have_in;
-				//artsdebug("requesting %d\n", need_more);
-				/* when there is no source (constant input value), then
-					we don't need to request something, it's just that
-					it can't supply more because the buffer isn't big enough
-				*/
-				if(inConn[in]->sourcemodule)
-					inConn[in]->sourcemodule->request(need_more);
-
-				have_in = inConn[in]->haveIn();
-	
-				//artsdebug("now connection %d indicates to have %d\n",in,have_in);
-				if(CanPerform > have_in) CanPerform = have_in;
-			}
+			if(inConn[c]->source || inConn[c]->gslIsConstant)
+				result++;
 		}
-		have_done = calc(CanPerform);
-		have_done_total += have_done;
+	}
 
-		/*
-		if(dsp->m_BusyHit != 0) artsdebug("got busyhit: %s\n",dsp->m_Name);
-		*/
-
-	} while(BusyHit && NeedCycles != CanPerform && have_done);
-	/* makes only sense to repeat if
-		 - there was a busyhit which indicates we are in a feedback loop
-		 - we should have done more than we have done
-         - actually something could be calculated
-     */
-	Busy = 0;
-	return(have_done);
+	return result;
 }
 
-/* This routine would now actually let the plugin calculate some data
-   that means generate sinus waves, mix audio signals etc.
-
-   The number of cycles is guaranteed to work without input underrun
-   by the flow system. But the routine still needs to check output
-   stall situations (ring buffer full).
-*/
-
-unsigned long StdScheduleNode::calc(unsigned long cycles)
+unsigned long StdScheduleNode::outputConnectionCount(const string& port)
 {
-	unsigned long i,room;
+	unsigned long result = 0;
 
-	//cout << "ca" << module->_interfaceName() << endl;
-	/* output sanity check:
-	   when there is not enough room in one of the buffers, we
-	   can't calculate that much cycles
-	*/
-	for(i=0;i<outConnCount;i++)
+	unsigned int c;
+	for(c = 0; c < outConnCount; c++)
 	{
-		room = outConn[i]->outRoom();
-		if(room < cycles)
-		{
-			cycles = room;
-			/*
-			artsdebug("- reduced calculation to %d due to lack of space\n",cycles);
-			*/
-		}
+		if(outConn[c]->name() == port)
+			result += outConn[c]->destcount;
 	}
 
-	if(cycles == 0) return(0);
-
-	//artsdebug("DSP %s: calculation of %d cycles\n",getClassName(),cycles);
-	/* input sanity check:
-		it's guaranteed that we have enough input, but do the check
-		anyway... - you never know
-	*/
-	for(i=0;i<inConnCount;i++)
-	{
-		/* otherwise input is "overconsumed" */
-		assert(inConn[i]->haveIn() >= cycles);
-
-		/* check sanity of the needread setting:
-			either data comes from fixed value (sourcemodule not assigned)
-			then we can't expect needread to contain sensible setting,
-			but otherwise: needread should have expected that we'll want
-			to read the data and thus be more than the cycles! */
-
-		assert((!inConn[i]->sourcemodule)
-				|| (inConn[i]->buffer->needread >= cycles));
-	}
-
-	unsigned long j, donecycles = 0, cando = 0;
-	
-	while(donecycles != cycles)
-	{	
-		cando = cycles-donecycles;
-
-		for(j=0;j<inConnCount;j++)
-			cando = inConn[j]->readSegment(donecycles,cando);
-
-		for(j=0;j<outConnCount;j++)
-			cando = outConn[j]->writeSegment(donecycles,cando);
-
-		module->calculateBlock(cando);
-		donecycles += cando;
-	}
-	assert(donecycles == cycles);
-
-	// actually update buffer by subtracting consumed input
-	for(i=0;i<inConnCount;i++)
-		inConn[i]->read(cycles);
-
-	// and adding fresh output
-	for(i=0;i<outConnCount;i++)
-		outConn[i]->write(cycles);
-
-	NeedCycles -= cycles;
-	CanPerform -= cycles;
-	return(cycles);
+	return result;
 }
 
 StdFlowSystem::StdFlowSystem()
@@ -994,11 +830,20 @@ StdFlowSystem::StdFlowSystem()
 	static bool gsl_is_initialized = false;
 	if(!gsl_is_initialized)
 	{
+		GslConfigValue values[2] = { { "wave_chunk_padding", 8 }, { 0, 0 } };
 		gsl_is_initialized = true;
 
 		g_thread_init(0);
-		gsl_init(0, (GslMutexTable *)gslGlobalMutexTable);
-		gsl_engine_init(false, 512, 44100);
+		gsl_init(values, (GslMutexTable *)gslGlobalMutexTable);
+
+		/*
+		 * FIXME: both of these are really supposed to be tunable
+		 * - the 512 because of low-latency apps, where calculating smaller
+		 *   block sizes might be necessary
+		 * - the 44100 because of the obvious reason, that not every artsd
+		 *   is running at that rate
+		 */
+		gsl_engine_init(false, 512, 44100, /* subsamplemask */ 63);
 		if(gslGlobalMutexTable)
 			arts_debug("gsl: using Unix98 pthreads directly for mutexes and conditions");
 		/*gsl_engine_debug_enable(GslEngineDebugLevel(GSL_ENGINE_DEBUG_JOBS | GSL_ENGINE_DEBUG_SCHED));*/
@@ -1102,7 +947,7 @@ void StdFlowSystem::connectObject(Object sourceObject,const string& sourcePort,
 		sn->connect(sourcePort,destsn,destPort);
 		return;
 	}
-	
+
 	ASyncPort *ap = port->asyncPort();
 
 	if(ap)
@@ -1217,58 +1062,6 @@ void StdFlowSystem::startedChanged()
 	needUpdateStarted = true;
 }
 
-void StdFlowSystem::schedule(unsigned long samples)
-{
-/** new style (dynamic size) scheduling **/
-
-	unsigned long MCount = nodes.size();
-	unsigned long *done = (unsigned long *)calloc(MCount,sizeof(unsigned long));
-	unsigned long i;
-	std::list<StdScheduleNode*>::iterator ni;
-	long incomplete, died = 0;
-
-	//printf("entering; samples = %d\n",samples);
-	do {
-		incomplete = 0;		/* assume we have calculated all samples for all
-								consumers, and later increment if some are
-								still missing */
-		for(ni = nodes.begin(), i = 0; ni != nodes.end(); ni++, i++)
-		{
-			StdScheduleNode *node = *ni;
-
-            unsigned int outConnections = 0,c;
-            for(c=0;c<node->outConnCount && !outConnections;c++)
-                outConnections += node->outConn[c]->destcount;                  
-
-			if(outConnections == 0 && node->running)
-			{
-				//printf("consumber = %s,done = %d, samples = %d\n",SynthModules[i]->getClassName(),done[i],samples);
-				/* a module whose input is not comsumed from other modules
-					is a "push delivery" style module, such as speakers,
-					or writing audio to log file, etc. and has to get
-					external requests from the scheduling system */
-
-				if(done[i] != samples)
-					done[i] += node->request(samples-done[i]);
-				assert(done[i] <= samples);
-				if(done[i] != samples) incomplete++;
-				//printf("*scheduler*\n");
-				died ++;
-				if(died > 10000)
-				{
-					free(done);
-					arts_warning("scheduler confusion: circle?");
-					return;
-				}
-			}
-		}
-	} while(incomplete);
-
-	//printf("<=> done!!\n");
-	free(done);
-	//return true;
-}
-
 // hacked initialization of Dispatcher::the()->flowSystem ;)
 
 namespace Arts {
