@@ -21,10 +21,13 @@
     */
 
 #include "stdsynthmodule.h"
+#include "synthschedule.h"
 #include "audiosubsys.h"
 #include "flowsystem.h"
+#include "debug.h"
 
 using namespace Arts;
+using namespace std;
 
 StdSynthModule::StdSynthModule()
 {
@@ -60,3 +63,30 @@ AutoSuspendState StdSynthModule::autoSuspend()
 {
 	return asNoSuspend;
 }
+
+unsigned long StdSynthModule::inputConnectionCount(const std::string& port)
+{
+	StdScheduleNode *xnode = (StdScheduleNode *)
+		_node()->cast("StdScheduleNode");
+	arts_return_val_if_fail(xnode, 0);
+
+	return xnode->inputConnectionCount(port);
+}
+
+unsigned long StdSynthModule::outputConnectionCount(const std::string& port)
+{
+	StdScheduleNode *xnode = (StdScheduleNode *)
+		_node()->cast("StdScheduleNode");
+	arts_return_val_if_fail(xnode, 0);
+
+	return xnode->outputConnectionCount(port);
+}
+
+bool StdSynthModule::connectionCountChanged()
+{
+	StdScheduleNode *xnode = (StdScheduleNode *)
+		_node()->cast("StdScheduleNode");
+	arts_return_val_if_fail(xnode, 0);
+
+	return xnode->connectionCountChanged();
+}
