@@ -455,6 +455,8 @@ Buffer *Dispatcher::waitForResult(long requestID, Connection *connection)
 {
 	bool isMainThread = SystemThreads::the()->isMainThread();
 	Buffer *b = requestResultPool[requestID];
+	
+	connection->_copy(); // Keep extra ref
 
 	while(!b && !connection->broken()) {
 		if(isMainThread)
@@ -468,9 +470,11 @@ Buffer *Dispatcher::waitForResult(long requestID, Connection *connection)
 	requestResultPool.releaseSlot(requestID);
 
 	if(connection->broken()) // connection went away before we got some result
-		return 0;
-	else
-		return b;
+		b = 0;
+	
+	connection->_release(); // Give up extra ref
+	
+	return b;
 }
 
 Buffer *Dispatcher::createRequest(long& requestID, long objectID, long methodID)
@@ -795,10 +799,12 @@ void Dispatcher::removeObject(long objectID)
 
 void Dispatcher::generateServerID()
 {
-	char buffer[4096];
-	sprintf(buffer,"%s-%04x-%08lx",MCOPUtils::getFullHostname().c_str(),
-									getpid(),time(0));
+	char *buffer;
+	buffer = arts_strdup_printf("%s-%04x-%08lx",
+								MCOPUtils::getFullHostname().c_str(),
+								getpid(),time(0));
 	serverID = buffer;
+	free(buffer);
 }
 
 string Dispatcher::objectToString(long objectID)
@@ -916,6 +922,7 @@ Connection *Dispatcher::connectUrl(const string& url)
 
 	if(conn)
 	{
+		conn->_copy(); // Keep extra ref for when the connection breaks
 		conn->setConnState(Connection::expectServerHello);
 
 		while((conn->connState() != Connection::established)
@@ -930,10 +937,13 @@ Connection *Dispatcher::connectUrl(const string& url)
 		if(conn->connState() == Connection::established)
 		{
 			connections.push_back(conn);
+			conn->_release(); // Give up extra ref
 			return conn;
 		}
 			
 		// well - bad luck (building a connection failed)
+		
+		// Give up extra ref
 		conn->_release();
 	}
 	return 0;
